package protocolP2P;
import protocolP2P.Payload;
import protocolP2P.RequestResponseCode;
import exception.TransmissionError;
import exception.ProtocolError;
import exception.InternalError;
import exception.SizeError;
import tools.BytesArrayTools;
import java.io.UnsupportedEncodingException;

/** Representation of payload for load request.
 * @author	Louis Royer
 * @author	Flavien Haas
 * @author	JS Auge
 * @version	1.0
 */
public class LoadRequest extends Payload {
	private String filename;
	private long maxSizePartialContent;
	private long offset;
	static final private int OFFSET_POSITION = 8;
	static final private int MAX_SIZE_PARTIAL_CONTENT_POSITION = OFFSET_POSITION + 8;
	static final private int FILENAME_SIZE_POSITION = MAX_SIZE_PARTIAL_CONTENT_POSITION + 8;
	static final private int FILENAME_POSITION = FILENAME_SIZE_POSITION + 4;

	/** Constructor (typically used by the server) with a filename parameter.
	 * @param filename	name of the file to download. Must not be empty.
	 * @param offset	offset of the bloc
	 * @param maxSizePartialContent	partial content in response should not excess this size, but this can be less (by example if endoffile is reached or server doesn't have the full block requested)
	 * @throws	InternalError
	 */
	public LoadRequest(String filename, long offset, long maxSizePartialContent) throws InternalError {
		super(RequestResponseCode.LOAD_REQUEST);
		/* assert to help debugging */
		assert filename.length() != 0 : "Payload size of LoadRequest must not be empty";
		if (filename.length() == 0) {
			throw new InternalError();
		}
		this.filename = filename;
		this.maxSizePartialContent = maxSizePartialContent;
		this.offset = offset;
	}

	/** Constructor (typically used by client) with a byte[] parameter containing the datagram received.
	 * @param datagram	the full datagram received
	 * @throws	SizeError
	 * @throws	InternalError
	 * @throws	ProtocolError
	 * @throws	TransmissionError
	 */
	protected LoadRequest(byte[] datagram) throws TransmissionError, SizeError, ProtocolError, InternalError {
		super(datagram);
		/* assert to help debugging */
		assert requestResponseCode == RequestResponseCode.LOAD_REQUEST : "LoadRequest subclass is incompatible with this datagram, request/response code must be checked before using this constructor";
		/* InternalErrorException */
		if (requestResponseCode!= RequestResponseCode.LOAD_REQUEST) {
			throw new InternalError();
		}
		/* Read offset */
		offset = BytesArrayTools.readLong(datagram, OFFSET_POSITION);

		/* Read maxSizePartialContent */
		maxSizePartialContent = BytesArrayTools.readLong(datagram, MAX_SIZE_PARTIAL_CONTENT_POSITION);
		
		/* Read filename */
		int size = BytesArrayTools.readInt(datagram, FILENAME_SIZE_POSITION);
		try {
			filename = new String(datagram, FILENAME_POSITION, size, "UTF-8");
		} catch (UnsupportedEncodingException e) {
			throw new InternalError();
		}
	}

	/** Returns a byte[] containing datagram with padding.
	 * This datagram is still incomplete and should not be send directly.
	 * ProtocolP2PDatagram will use this method to generate the complete datagram.
	 * @return datagram with padding
	 * @throws	InternalError
	 */
	protected byte[] toDatagram() throws InternalError {
		// compute size
		int filenameSize = filename.length();
		int size = FILENAME_POSITION + filenameSize;
		byte[] datagram = new byte[size + 1]; // java initialize all to zero
		// set request/response code
		datagram[RequestResponseCode.RRCODE_POSITION] = requestResponseCode.codeValue;
		// bits 16-31 are reserved for future use
		// 8 bytes for headers
		setPayloadSize(size - OFFSET_POSITION, datagram);

		// Write offset
		BytesArrayTools.write(datagram, OFFSET_POSITION, (long)offset);

		// Write maxSizePartialContent
		BytesArrayTools.write(datagram, MAX_SIZE_PARTIAL_CONTENT_POSITION, (long)maxSizePartialContent);

		// Write filenameSize
		BytesArrayTools.write(datagram, FILENAME_SIZE_POSITION, (int)filenameSize);
		
		// Write filename
		int bCount = FILENAME_POSITION;
		try {
			byte[] sb = filename.getBytes("UTF-8");
			for(byte b : sb) {
				datagram[bCount] = b;
				bCount += 1;
			}
		} catch (UnsupportedEncodingException e) {
			throw new InternalError();
		}
		return datagram;
	}

	/** filename getter.
	 * @return	filename
	 */
	public String getFilename() {
		return filename;
	}

	/** offset getter.
	 * @return	offset
	 */
	public long getOffset() {
		return offset;
	}

	/** maxSizePartialContent getter.
	 * @return	maxSizePartialContent
	 */
	public long getMaxSizePartialContent() {
		return maxSizePartialContent;
	}
}
